==> api-utils-7.0.2.tgz: package/src/ws/server/trpc/awsLambdaTrpcSubscriptionRequestHandler.ts <==
import {GetConnectionCommand, GoneException, PostToConnectionCommand} from '@aws-sdk/client-apigatewaymanagementapi'
import {type AnyTRPCRouter, callTRPCProcedure, type inferRouterContext} from '@trpc/server'
import type {APIGatewayProxyEvent, APIGatewayProxyResult, Context} from 'aws-lambda'
import {getApiGatewayManagementApiClient} from '../getApiGatewayManagementApiClient'
import {getConnection} from '../getConnection'
import type {TrpcSubscriptionRequest} from './TrpcSubscriptionRequest'
import type {TrpcSubscriptionResponse} from './TrpcSubscriptionResponse'

export const awsLambdaTrpcSubscriptionRequestHandler = <Router extends AnyTRPCRouter>(opts: {
  router: Router
  createContext?: (event: APIGatewayProxyEvent) => inferRouterContext<Router> | Promise<inferRouterContext<Router>>
}) => {
  const {router, createContext} = opts

  return async (event: APIGatewayProxyEvent, context: Context): Promise<APIGatewayProxyResult> => {
    context.callbackWaitsForEmptyEventLoop = false
    const connection = getConnection(event)

    if (!event.body || event.body === '[]') return {statusCode: 200, body: '{}'}

    const requestOrRequests = JSON.parse(event.body) as TrpcSubscriptionRequest | TrpcSubscriptionRequest[]

    const requests = Array.isArray(requestOrRequests) ? requestOrRequests : [requestOrRequests]

    await Promise.all(
      requests.map(async (request) => {
        const sendMessage = async (response: TrpcSubscriptionResponse) => {
          await getApiGatewayManagementApiClient(connection.endpoint).send(
            new PostToConnectionCommand({
              ConnectionId: connection.connectionId,
              Data: JSON.stringify(response),
            }),
          )
        }
        const isConnected = async (): Promise<boolean> => {
          return getApiGatewayManagementApiClient(connection.endpoint)
            .send(new GetConnectionCommand({ConnectionId: connection.connectionId}))
            .then(() => true)
            .catch(() => false)
        }
        const stopConnection = async (): Promise<void> => {
          await sendMessage({
            id: request.id,
            result: {type: 'stopped'},
          })
        }

        try {
          if (request.method === 'subscription.stop') {
            await stopConnection()
          }

          if (request.method === 'subscription') {
            await sendMessage({
              id: request.id,
              result: {type: 'started'},
            })

            const procedure: AsyncGenerator = await callTRPCProcedure({
              _def: router._def,
              path: request.params.path,
              type: 'subscription',
              ctx: (await createContext?.(event)) || {},
              getRawInput: async () => request.params.input,
              input: request.params.input,
              signal: undefined,
            })

            try {
              // TODO: ping still needed?
              const pingIntervalMs = 5_000
              let lastPing = Date.now()
              for await (const data of procedure) {
                if (Date.now() > lastPing + pingIntervalMs) {
                  if (!(await isConnected())) {
                    console.info('Ping: Connection closed by client')
                    return
                  }
                  lastPing = Date.now()
                }

                await sendMessage({
                  id: request.id,
                  result: {
                    type: 'data',
                    data: data,
                  },
                })
              }

              await sendMessage({
                id: request.id,
                result: {type: 'stopped'},
              })
            } catch (error: unknown) {
              console.error(error)
              await sendMessage({
                id: request.id,
                result: {type: 'stopped'},
              })
            }
          }
        } catch (err) {
          if (err instanceof GoneException) {
            console.info('Error: Connection closed by client', err.message)
          } else {
            console.error(err)
            await stopConnection()
          }
        }
      }),
    )

    return {
      statusCode: 200,
      body: '{}',
    }
  }
}

==> api-utils-7.0.3.tgz: package/src/ws/server/trpc/awsLambdaTrpcSubscriptionRequestHandler.ts <==
import {GetConnectionCommand, GoneException, PostToConnectionCommand} from '@aws-sdk/client-apigatewaymanagementapi'
import {type AnyTRPCRouter, callTRPCProcedure, type inferRouterContext} from '@trpc/server'
import type {APIGatewayProxyEvent, APIGatewayProxyResult, Context} from 'aws-lambda'
import {getApiGatewayManagementApiClient} from '../getApiGatewayManagementApiClient.ts'
import {getConnection} from '../getConnection.ts'
import type {TrpcSubscriptionRequest} from './TrpcSubscriptionRequest.ts'
import type {TrpcSubscriptionResponse} from './TrpcSubscriptionResponse.ts'

export const awsLambdaTrpcSubscriptionRequestHandler = <Router extends AnyTRPCRouter>(opts: {
  router: Router
  createContext?: (event: APIGatewayProxyEvent) => inferRouterContext<Router> | Promise<inferRouterContext<Router>>
}) => {
  const {router, createContext} = opts

  return async (event: APIGatewayProxyEvent, context: Context): Promise<APIGatewayProxyResult> => {
    context.callbackWaitsForEmptyEventLoop = false
    const connection = getConnection(event)

    if (!event.body || event.body === '[]') return {statusCode: 200, body: '{}'}

    const requestOrRequests = JSON.parse(event.body) as TrpcSubscriptionRequest | TrpcSubscriptionRequest[]

    const requests = Array.isArray(requestOrRequests) ? requestOrRequests : [requestOrRequests]

    await Promise.all(
      requests.map(async (request) => {
        const sendMessage = async (response: TrpcSubscriptionResponse) => {
          await getApiGatewayManagementApiClient(connection.endpoint).send(
            new PostToConnectionCommand({
              ConnectionId: connection.connectionId,
              Data: JSON.stringify(response),
            }),
          )
        }
        const isConnected = async (): Promise<boolean> => {
          return getApiGatewayManagementApiClient(connection.endpoint)
            .send(new GetConnectionCommand({ConnectionId: connection.connectionId}))
            .then(() => true)
            .catch(() => false)
        }
        const stopConnection = async (): Promise<void> => {
          await sendMessage({
            id: request.id,
            result: {type: 'stopped'},
          })
        }

        try {
          if (request.method === 'subscription.stop') {
            await stopConnection()
          }

          if (request.method === 'subscription') {
            await sendMessage({
              id: request.id,
              result: {type: 'started'},
            })

            const procedure: AsyncGenerator = await callTRPCProcedure({
              router,
              path: request.params.path,
              type: 'subscription',
              ctx: (await createContext?.(event)) || {},
              getRawInput: async () => request.params.input,
              input: request.params.input,
              signal: undefined,
            })

            try {
              // TODO: ping still needed?
              const pingIntervalMs = 5_000
              let lastPing = Date.now()
              for await (const data of procedure) {
                if (Date.now() > lastPing + pingIntervalMs) {
                  if (!(await isConnected())) {
                    console.info('Ping: Connection closed by client')
                    return
                  }
                  lastPing = Date.now()
                }

                await sendMessage({
                  id: request.id,
                  result: {
                    type: 'data',
                    data: data,
                  },
                })
              }

              await sendMessage({
                id: request.id,
                result: {type: 'stopped'},
              })
            } catch (error: unknown) {
              console.error(error)
              await sendMessage({
                id: request.id,
                result: {type: 'stopped'},
              })
            }
          }
        } catch (err) {
          if (err instanceof GoneException) {
            console.info('Error: Connection closed by client', err.message)
          } else {
            console.error(err)
            await stopConnection()
          }
        }
      }),
    )

    return {
      statusCode: 200,
      body: '{}',
    }
  }
}
